﻿Option Strict On

Imports System
Imports System.IO
Imports System.Text
Imports System.Threading
Imports System.Diagnostics
Imports System.Globalization

Namespace Logging

#Region "Enums"

    Public Enum SwitchReasons
        None = 0
        Size = 1
        Day = 2
        Week = 3
        Month = 4
    End Enum

    Public Enum OutputFormat
        [Default] = 0
        Fixed = 1
        CSV = 2
        ExcelCSV = 3
    End Enum

#End Region


    Public Class BlogTraceListener : Inherits TraceListener

#Region "Private Konstanten"

        Private Const defaultInitDataPath As String = "C:\Temp\"
        Private Const defaultInitDataFileName As String = "LogFile-{0}-{1}-{2:D8}-{3:D8}"
        Private Const defaultInitDataFileExtention As String = "log"
        Private Const defaultInitDataSwitchReason As String = "T"
        Private Const defaultInitDataOutputFormat As String = "FIX"

        Private Const defaultInitData As String = _
            defaultInitDataPath + _
            defaultInitDataFileName + "." + _
            defaultInitDataFileExtention + ";" + _
            defaultInitDataSwitchReason + "," + _
            defaultInitDataOutputFormat

        Private Const maxCategoryNameLength As Integer = 12
        Private ReadOnly messageFormatStringFixedFirstLine As String = String.Format("{{0,-{0}:C}} {{1:0000}} {{2:00000}} {{3:00000}} {{4}} {{5}}", maxCategoryNameLength)
        Private ReadOnly messageFormatStringFixedLines As String = String.Format(String.Format("{{0,-{0}:C}}", maxCategoryNameLength + 38), " ") + "{0}"
        Private ReadOnly messageFormatStringCSV As String = "{0},{1},{2},{3},{4},{5}"

        Private Const defaultCategory As String = "NONE"
        Private Const defaultCategoryTrace As String = "TRACE"

        Private Const parameterSwitchReasonDay As String = "D"
        Private Const parameterSwitchReasonTag As String = "T"
        Private Const parameterSwitchReasonWeek As String = "W"
        Private Const parameterSwitchReasonMonth As String = "M"

        Private Const parameterOutputFormatFixed As String = "FIX"
        Private Const parameterOutputFormatCSV As String = "CSV"
        Private Const parameterOutputFormatExcelCSV As String = "EXCSV"

#End Region

#Region "Private Variablen"

        Private switchReason As SwitchReasons = SwitchReasons.None
        Private outputFormat As OutputFormat = outputFormat.Default

        Private logFileIsOpen As Boolean = False
        Private logFileName As String = Nothing
        Private logFileExtention As String = Nothing
        Private logFileMaxLength As Long = 0
        Private logFileInfo As FileInfo = Nothing
        Private logFileStream As StreamWriter = Nothing
        Private logFileDateTime As DateTime

        Private loggedMessages As Long = 0

#End Region

#Region "Konstruktor(en)"

        ''' <summary>
        ''' Constructor
        ''' </summary>
        Public Sub New()
            Me.New(defaultInitData)
        End Sub

        ''' <summary>
        ''' Constructor
        ''' </summary>
        ''' <param name="initializeData">Initialisierungsparameter</param>
        Public Sub New(initializeData As String)
            initData(initializeData)
        End Sub

#End Region

#Region "Destuktor"

        ''' <summary>
        ''' Destructor 
        ''' </summary>
        Protected Overrides Sub Finalize()
            WriteLine("Trace beendet", defaultCategoryTrace)
        End Sub

#End Region

#Region "Private Methoden"

        ''' <summary>
        ''' Initialisierung der Variablen, Schreiben der Header-Informationen
        ''' </summary>
        ''' <param name="initializeData">Initialisierungs-String (z. B. aus app.config)</param>
        Private Sub initData(initializeData As String)
            If (initializeData Is Nothing Or initializeData.Length = 0) Then
                initializeData = defaultInitData
            End If

            Dim tempArray As String()
            Dim delimiterArray As Char() = New Char() {";"c, ","c}
            Dim tempFilename As StringBuilder

            Dim currentProcess As Process = Process.GetCurrentProcess()

            tempArray = initializeData.Split(delimiterArray, 3)

            If (tempArray.Length > 0) Then
                tempFilename = New StringBuilder()
            Else
                tempFilename = New StringBuilder(defaultInitDataPath)
            End If

            tempFilename.AppendFormat(tempArray(0), _
                System.Environment.MachineName, _
                System.Environment.UserName, _
                currentProcess.Id, _
                currentProcess.SessionId)
            logFileName = tempFilename.ToString()

            logFileName = Environment.ExpandEnvironmentVariables(logFileName)
            logFileExtention = Path.GetExtension(logFileName)

            logFileName = logFileName.Replace(logFileExtention, "")
            logFileExtention = logFileExtention.Replace(".", "")

            If (logFileExtention = "") Then
                logFileExtention = defaultInitDataFileExtention
            End If

            logFileMaxLength = 0
            If (tempArray.Length > 1) Then
                Select Case (tempArray(1).Trim().ToUpper())
                    Case parameterSwitchReasonDay
                    Case parameterSwitchReasonTag
                        switchReason = SwitchReasons.Day
                    Case parameterSwitchReasonWeek
                        switchReason = SwitchReasons.Week
                    Case parameterSwitchReasonMonth
                        switchReason = SwitchReasons.Month
                    Case Else
                        switchReason = SwitchReasons.None
                        If (Int64.TryParse(tempArray(1), logFileMaxLength)) Then
                            If (logFileMaxLength > 0) Then
                                switchReason = SwitchReasons.Size
                            End If
                        End If
                End Select
            Else
                switchReason = SwitchReasons.None
            End If

            If (tempArray.Length > 2) Then
                Select Case (tempArray(2).Trim().ToUpper())
                    Case parameterOutputFormatFixed
                        outputFormat = outputFormat.Fixed
                    Case parameterOutputFormatCSV
                        outputFormat = outputFormat.CSV
                    Case parameterOutputFormatExcelCSV
                        outputFormat = outputFormat.ExcelCSV
                    Case Else
                        outputFormat = outputFormat.Default
                End Select
            Else
                outputFormat = outputFormat.Default
            End If

            logFileInfo = New FileInfo(logFileName + "." + logFileExtention)

            If (logFileInfo.Exists) Then
                logFileDateTime = logFileInfo.CreationTime
            Else
                logFileDateTime = DateTime.Now
            End If

            fileOpen()

#If DEBUG Then
            WriteLine("Trace gestartet - DebugVersion", defaultCategoryTrace)
#Else
            writeLine("Trace gestartet - ReleaseVersion", defaultCategoryTrace)
#End If
            WriteLine("initializeData=""" + initializeData + " """, defaultCategoryTrace)

            Dim parameterSwitchReason As String = ""
            Select Case (switchReason)
                Case SwitchReasons.None
                    parameterSwitchReason = "Kein Dateiwechsel"
                Case SwitchReasons.Size
                    parameterSwitchReason = String.Format("Dateiwechsel bei Dateigröße von {0} bytes", logFileMaxLength)
                Case SwitchReasons.Day
                    parameterSwitchReason = "Dateiwechsel täglich"
                Case SwitchReasons.Week
                    parameterSwitchReason = "Dateiwechsel wöchentlich"
                Case SwitchReasons.Month
                    parameterSwitchReason = "Dateiwechsel monatlich"
            End Select

            WriteLine(parameterSwitchReason, defaultCategoryTrace)

            writeHeader()

            fileClose()
        End Sub

        ''' <summary>
        ''' Öffnet die Ausgabedatei, sofern diese noch nicht geöffnet ist
        ''' </summary>
        Private Sub fileOpen()
            If (Not logFileIsOpen) Then
                logFileStream = logFileInfo.AppendText()
                logFileStream.AutoFlush = True
                logFileIsOpen = True
            End If
        End Sub

        ''' <summary>
        ''' Schließt die Ausgabedatei, sofern diese geöffnet ist
        ''' </summary>
        Private Sub fileClose()
            If (logFileIsOpen) Then
                logFileStream.Close()
                logFileIsOpen = False
            End If
        End Sub

        ''' <summary>
        ''' Verschiebt die Ausgabedatei beim Dateiwechsel
        ''' </summary>
        ''' <param name="time">Zeitstempel für den Dateiwechsel</param>
        ''' <returns>Dateiname der alten Ausgabedatei</returns>
        Private Function fileMove(time As DateTime) As String
            Dim moveFileName As StringBuilder

            moveFileName = New StringBuilder()
            moveFileName.Append(logFileName)
            moveFileName.AppendFormat("-{0}.{1}", _
                time.ToString("yyyy-MM-dd-HH-mm"), _
                logFileExtention)

            logFileDateTime = time

            fileClose()

            logFileInfo.MoveTo(moveFileName.ToString())

            Return moveFileName.ToString()
        End Function

        ''' <summary>
        ''' Testet, ob eine der Bedingungen für Ausgabedateiwechsel erfüllt ist 
        ''' und führt  ggf. den Dateiwechsel aus
        ''' </summary>
        Private Sub checkSwitchReason()
            Dim switchFlag As Boolean
            Dim reason As String = Nothing
            Dim time As DateTime = DateTime.Now

            logFileInfo.Refresh()

            switchFlag = False
            Select Case (switchReason)
                Case SwitchReasons.Size
                    If (logFileInfo.Length >= logFileMaxLength And logFileMaxLength <> 0) Then
                        switchFlag = True
                        reason = "Dateiwechsel: Dateilänge überschritten"
                    End If

                Case SwitchReasons.Day
                    If (logFileDateTime.DayOfYear <> time.DayOfYear) Then
                        switchFlag = True
                        reason = "Dateiwechsel: Tagesgrenze überschritten"
                    End If

                Case SwitchReasons.Week
                    Dim culture As CultureInfo = System.Threading.Thread.CurrentThread.CurrentCulture
                    Dim cal As Calendar = culture.Calendar
                    Dim weekFile As Integer = cal.GetWeekOfYear(logFileDateTime, culture.DateTimeFormat.CalendarWeekRule, culture.DateTimeFormat.FirstDayOfWeek)
                    Dim week As Integer = cal.GetWeekOfYear(time, culture.DateTimeFormat.CalendarWeekRule, culture.DateTimeFormat.FirstDayOfWeek)

                    If (weekFile <> week) Then
                        switchFlag = True
                        reason = "Dateiwechsel: Wochengrenze überschritten"
                    End If

                Case SwitchReasons.Month
                    If (logFileDateTime.Month <> time.Month) Then
                        switchFlag = True
                        reason = "Dateiwechsel: Monatsgrenze überschritten"
                    End If
                Case Else
            End Select

            If (switchFlag) Then
                writeSwitchReason(reason)

                Dim oldFileName As String = fileMove(time)
                logFileInfo = Nothing
                logFileInfo = New FileInfo(logFileName + "." + logFileExtention)

                fileOpen()

                writeSwitchReason(reason)
                WriteLine("Dateiwechsel, alte Datei unter " + oldFileName.ToString(), defaultCategoryTrace)
                writeHeader()
            End If
        End Sub

        ''' <summary>
        ''' Schreibt Basisinformationen in die Ausgabedatei
        ''' </summary>
        Public Sub writeHeader()
            Dim isOpen As Boolean = logFileIsOpen
            fileOpen()

            Dim process As Process = process.GetCurrentProcess()

            WriteLine("Assembly:  " + Me.GetType().Assembly.GetName().Version.ToString(), defaultCategoryTrace)
            WriteLine("Framework: " + Environment.Version.ToString(), defaultCategoryTrace)
            WriteLine("Anwender:  " + Environment.UserName, defaultCategoryTrace)
            WriteLine("Domain:    " + Environment.UserDomainName, defaultCategoryTrace)
            WriteLine("Hostname:  " + Environment.MachineName, defaultCategoryTrace)
            WriteLine("-------------------------------------------------------------------------", defaultCategoryTrace)
            WriteLine("Thread:    " + Thread.CurrentThread.ManagedThreadId.ToString(), defaultCategoryTrace)
            WriteLine("Process:   " + Process.Id.ToString(), defaultCategoryTrace)
            WriteLine("Session:   " + Process.SessionId.ToString(), defaultCategoryTrace)
            WriteLine("-------------------------------------------------------------------------", defaultCategoryTrace)
            WriteLine("", defaultCategoryTrace)

            If (Not isOpen) Then fileClose()
        End Sub

        ''' <summary>
        ''' Schreibt Informationen zum Dateiwechsel in die Ausgabedatei
        ''' </summary>
        ''' <param name="switchReason"></param>
        Private Sub writeSwitchReason(switchReason As String)
#If DEBUG Then
            WriteLine(switchReason + " - DebugVersion", defaultCategoryTrace)
#Else
            writeLine(switchReason + " - ReleaseVersion", defaultCategoryTrace)
#End If
            WriteLine("In der aktuellen Session wurden bisher " + loggedMessages.ToString() + " Nachrichten geschrieben", defaultCategoryTrace)
        End Sub

        ''' <summary>
        ''' Schreibt eine Nachricht und zusätzliche Ereignisinformationen mit Kategorie in die Ausgabedatei.
        ''' </summary>
        ''' <param name="message">Eine zu schreibende Meldung.</param>
        ''' <param name="category">Ein Kategoriename für die Zuordnung der Ausgabe.</param>
        ''' <param name="id">Ein numerischer Bezeichner für das Ereignis.</param>
        ''' <param name="time">Datum und Uhrzeit, an dem das Ereignis aufgetreten ist.</param>
        ''' <param name="threadId">Eindeutiger Bezeichner für den aktuellen verwalteten Thread.</param>
        ''' <param name="processId">Eindeutiger Bezeichner des aktuellen Prozesses.</param>
        Private Overloads Sub writeLine(message As String, category As String, id As Long, time As DateTime, threadId As Integer, processId As Integer)
            fileOpen()

            checkSwitchReason()

            If (category.Length > maxCategoryNameLength) Then
                category = category.Substring(0, maxCategoryNameLength)
            End If

            Dim formatString As String
            If (outputFormat = outputFormat.CSV) Or (outputFormat = outputFormat.ExcelCSV) Then
                If (outputFormat = outputFormat.ExcelCSV) Then
                    formatString = messageFormatStringCSV.Replace(",", ";")
                Else
                    formatString = messageFormatStringCSV
                End If

                category = String.Format("""{0}""", category)
                If (message.IndexOfAny(New Char() {""""c, ","c, ";"c}) <> -1) Then
                    message = message.Replace("""", """""")
                End If
                message = String.Format("""{0}""", message)
            Else
                formatString = messageFormatStringFixedFirstLine
            End If

            If (outputFormat = outputFormat.CSV Or outputFormat = outputFormat.ExcelCSV) Then
                logFileStream.WriteLine(formatString, _
                    category.ToUpper(), _
                    id, _
                    threadId, _
                    processId, _
                    time.ToString("yyyy-MM-dd HH:mm:ss"), _
                    message)
            Else
                Dim lines As String() = message.Split(New String() {Environment.NewLine}, StringSplitOptions.None)
                Dim firstLine As Boolean = True

                For Each line As String In lines
                    If (firstLine) Then
                        logFileStream.WriteLine(formatString, _
                            category.ToUpper(), _
                            id, _
                            threadId, _
                            processId, _
                            time.ToString("yyyy-MM-dd HH:mm:ss"), _
                            line)
                        firstLine = False
                    Else
                        logFileStream.WriteLine(messageFormatStringFixedLines, line)
                    End If
                Next
            End If

            loggedMessages += 1

            fileClose()
        End Sub

#End Region

#Region "Public Overrides Methoden"

#Region "Write-Methode"

        ''' <summary>
        ''' Schreibt eine Nachricht in die Ausgabedatei.
        ''' </summary>
        ''' <param name="message">Eine zu schreibende Meldung.</param>
        ''' <remarks>Funktioniert analog zur <see cref="WriteLine"/> Methode. Ruft diese direkt auf.</remarks>
        Public Overrides Sub Write(message As String)
            Me.WriteLine(message)
        End Sub

        ''' <summary>
        ''' Schreibt eine Nachricht mit Kategorie in die Ausgabedatei.
        ''' </summary>
        ''' <param name="message">Eine zu schreibende Meldung.</param>
        ''' <param name="category">Ein Kategoriename für die Zuordnung der Ausgabe.</param>
        ''' <remarks>Funktioniert analog zur <see cref="WriteLine"/> Methode. Ruft diese direkt auf.</remarks>
        Public Overrides Sub Write(message As String, category As String)
            Me.WriteLine(message, category)
        End Sub

        ''' <summary>
        ''' Schreibt den Wert der ToString-Methode eines Objekts in die Ausgabedatei.
        ''' </summary>
        ''' <param name="o">Ein Object, dessen ToString-Methode geschrieben werden soll.</param>
        ''' <remarks>Funktioniert analog zur <see cref="WriteLine"/> Methode. Ruft diese direkt auf.</remarks>
        Public Overrides Sub Write(o As Object)
            Me.WriteLine(o)
        End Sub

        ''' <summary>
        ''' Schreibt den Wert der ToString-Methode eines Objekts mit Kategorie in die Ausgabedatei.
        ''' </summary>
        ''' <param name="o">Ein Object, dessen ToString-Methode geschrieben werden soll.</param>
        ''' <param name="category">Ein Kategoriename für die Zuordnung der Ausgabe.</param>
        ''' <remarks>Funktioniert analog zur <see cref="WriteLine"/> Methode. Ruft diese direkt auf.</remarks>
        Public Overrides Sub Write(o As Object, category As String)
            Me.WriteLine(o, category)
        End Sub

#End Region

#Region "WriteLine-Methode"

        ''' <summary>
        ''' Schreibt eine Nachricht in die Ausgabedatei.
        ''' </summary>
        ''' <param name="message">Eine zu schreibende Meldung.</param>
        ''' <remarks>Funktioniert analog zur <seeaslo cref="WriteLine"/> Methode. Ruft diese direkt auf.</remarks>

        Public Overrides Sub WriteLine(message As String)
            Me.WriteLine(message, defaultCategory)
        End Sub

        ''' <summary>
        ''' Schreibt eine Nachricht mit Kategorie in die Ausgabedatei.
        ''' </summary>
        ''' <param name="message">Eine zu schreibende Meldung.</param>
        ''' <param name="category">Ein Kategoriename für die Zuordnung der Ausgabe.</param>
        Public Overrides Sub WriteLine(message As String, category As String)
            Dim threadId As Integer = Thread.CurrentThread.ManagedThreadId
            Dim processId As Integer = Process.GetCurrentProcess().Id

            Me.writeLine(message, category, 0, DateTime.Now, threadId, processId)
        End Sub

        ''' <summary>
        ''' Schreibt den Wert der ToString-Methode eines Objekts in die Ausgabedatei.
        ''' </summary>
        ''' <param name="o">Ein Object, dessen ToString-Methode geschrieben werden soll.</param>
        Public Overrides Sub WriteLine(o As Object)
            Me.WriteLine(o, defaultCategory)
        End Sub

        ''' <summary>
        ''' Schreibt den Wert der ToString-Methode eines Objekts mit Kategorie in die Ausgabedatei.
        ''' </summary>
        ''' <param name="o">Ein Object, dessen ToString-Methode geschrieben werden soll.</param>
        ''' <param name="category">Ein Kategoriename für die Zuordnung der Ausgabe.</param>
        Public Overrides Sub WriteLine(o As Object, category As String)
            Me.WriteLine(o.ToString(), category)
        End Sub

#End Region

#Region "TraceData-Methode"

        ''' <summary>
        ''' Schreibt Ablaufverfolgungsinformationen, ein Datenobjekt und Ereignisinformationen in die spezifische Ausgabe des Listeners.
        ''' </summary>
        ''' <param name="eventCache">Ein TraceEventCache-Objekt mit der aktuellen Prozess-ID, Thread-ID und den aktuellen Stapelüberwachungsinformationen.</param>
        ''' <param name="source">Ein Name, der verwendet wird, um die Ausgabe zu bezeichnen, i. d. R. der Name der Anwendung, die das Ablaufverfolgungsereignis generiert hat.</param>
        ''' <param name="eventType">Einer der TraceEventType-Werte, der den Typ des Ereignisses angibt, das die Ablaufverfolgung verursacht hat.</param>
        ''' <param name="id">Ein numerischer Bezeichner für das Ereignis.</param>
        ''' <param name="data">Die auszugebenden Ablaufverfolgungsdaten.</param>
        ''' <remarks>Informationen </remarks>
        Public Overrides Sub TraceData(eventCache As TraceEventCache, source As String, eventType As TraceEventType, id As Integer, data As Object)
            Me.writeLine(source + ": " + data.ToString(), eventType.ToString(), id, eventCache.DateTime, Convert.ToInt32(eventCache.ThreadId), eventCache.ProcessId)
        End Sub

        ''' <summary>
        ''' Schreibt Ablaufverfolgungsinformationen, ein Array von Datenobjekten und Ereignisinformationen in die spezifische Ausgabe des Listeners.
        ''' </summary>
        ''' <param name="eventCache">Ein TraceEventCache-Objekt mit der aktuellen Prozess-ID, Thread-ID und den aktuellen Stapelüberwachungsinformationen.</param>
        ''' <param name="source">Ein Name, der verwendet wird, um die Ausgabe zu bezeichnen, i. d. R. der Name der Anwendung, die das Ablaufverfolgungsereignis generiert hat.</param>
        ''' <param name="eventType">Einer der TraceEventType-Werte, der den Typ des Ereignisses angibt, das die Ablaufverfolgung verursacht hat.</param>
        ''' <param name="id">Ein numerischer Bezeichner für das Ereignis.</param>
        ''' <param name="data">Ein Objektarray, das in Form von Daten ausgegeben werden soll.</param>
        Public Overrides Sub TraceData(eventCache As TraceEventCache, source As String, eventType As TraceEventType, id As Integer, ParamArray data As Object())
            Dim message As String = ""

            For Each obj As Object In data
                If (message <> "") Then message += ", "
                message += obj.ToString()
            Next
            Me.writeLine(source + ": " + message, eventType.ToString(), id, eventCache.DateTime, Convert.ToInt32(eventCache.ThreadId), eventCache.ProcessId)
        End Sub

#End Region

#Region "TraceEvent-Methode"

        ''' <summary>
        ''' Schreibt Ablaufverfolgungs- und Ereignisinformationen in die Ausgabedatei.
        ''' </summary>
        ''' <param name="eventCache">Ein TraceEventCache-Objekt mit der aktuellen Prozess-ID, Thread-ID und den aktuellen Stapelüberwachungsinformationen.</param>
        ''' <param name="source">Ein Name, der verwendet wird, um die Ausgabe zu bezeichnen, i. d. R. der Name der Anwendung, die das Ablaufverfolgungsereignis generiert hat.</param>
        ''' <param name="eventType">Einer der TraceEventType-Werte, der den Typ des Ereignisses angibt, das die Ablaufverfolgung verursacht hat.</param>
        ''' <param name="id">Ein numerischer Bezeichner für das Ereignis.</param>
        Public Overrides Sub TraceEvent(eventCache As TraceEventCache, source As String, eventType As TraceEventType, id As Integer)
            Me.TraceEvent(eventCache, source, eventType, id, "")
        End Sub

        ''' <summary>
        ''' Schreibt Ablaufverfolgungsinformationen, eine Meldung und Ereignisinformationen in die Ausgabedatei. 
        ''' </summary>
        ''' <param name="eventCache">Ein TraceEventCache-Objekt mit der aktuellen Prozess-ID, Thread-ID und den aktuellen Stapelüberwachungsinformationen.</param>
        ''' <param name="source">Ein Name, der verwendet wird, um die Ausgabe zu bezeichnen, i. d. R. der Name der Anwendung, die das Ablaufverfolgungsereignis generiert hat.</param>
        ''' <param name="eventType">Einer der TraceEventType-Werte, der den Typ des Ereignisses angibt, das die Ablaufverfolgung verursacht hat.</param>
        ''' <param name="id">Ein numerischer Bezeichner für das Ereignis.</param>
        ''' <param name="message">Eine zu schreibende Meldung.</param>
        Public Overrides Sub TraceEvent(eventCache As TraceEventCache, source As String, eventType As TraceEventType, id As Integer, message As String)
            If ((eventType = TraceEventType.Error Or eventType = TraceEventType.Critical) And eventCache.Callstack <> "") Then
                If (message <> "") Then message += Environment.NewLine
                message += eventCache.Callstack
            End If
            Me.writeLine(source + ": " + message, eventType.ToString(), id, eventCache.DateTime, Convert.ToInt32(eventCache.ThreadId), eventCache.ProcessId)
        End Sub

        ''' <summary>
        ''' Schreibt Ablaufverfolgungsinformationen, ein formatiertes Array von Objekten und Ereignisinformationen in die Ausgabedatei. 
        ''' </summary>
        ''' <param name="eventCache">Ein TraceEventCache-Objekt mit der aktuellen Prozess-ID, Thread-ID und den aktuellen Stapelüberwachungsinformationen.</param>
        ''' <param name="source">Ein Name, der verwendet wird, um die Ausgabe zu bezeichnen, i. d. R. der Name der Anwendung, die das Ablaufverfolgungsereignis generiert hat.</param>
        ''' <param name="eventType">Einer der TraceEventType-Werte, der den Typ des Ereignisses angibt, das die Ablaufverfolgung verursacht hat.</param>
        ''' <param name="id">Ein numerischer Bezeichner für das Ereignis.</param>
        ''' <param name="format">Eine Formatzeichenfolge mit 0 oder mehr Formatelementen, die Objekten im args-Array entsprechen.</param>
        ''' <param name="args">Ein object-Array mit 0 oder mehr zu formatierenden Objekten.</param>
        Public Overrides Sub TraceEvent(eventCache As TraceEventCache, source As String, eventType As TraceEventType, id As Integer, format As String, ParamArray args As Object())
            If (args IsNot Nothing) Then
                Me.TraceEvent(eventCache, source, eventType, id, String.Format(format, args))
            Else
                Me.TraceEvent(eventCache, source, eventType, id, format)
            End If
        End Sub

#End Region

#Region "TraceTransfer-Methode"

        ''' <summary>
        ''' Schreibt Ablaufverfolgungsinformationen, eine Meldung, eine verwandte Aktivitätsidentität und Ereignisinformationen in die Ausgabedatei.
        ''' </summary>
        ''' <param name="eventCache">Ein TraceEventCache-Objekt mit der aktuellen Prozess-ID, Thread-ID und den aktuellen Stapelüberwachungsinformationen.</param>
        ''' <param name="source">Ein Name, der verwendet wird, um die Ausgabe zu bezeichnen, i. d. R. der Name der Anwendung, die das Ablaufverfolgungsereignis generiert hat.</param>
        ''' <param name="id">Ein numerischer Bezeichner für das Ereignis.</param>
        ''' <param name="message">Eine zu schreibende Meldung.</param>
        ''' <param name="relatedActivityId">Ein Guid-Objekt, das eine verwandte Aktivität bezeichnet.</param>
        Public Overrides Sub TraceTransfer(eventCache As TraceEventCache, source As String, id As Integer, message As String, relatedActivityId As Guid)
            Dim eventType As TraceEventType = TraceEventType.Transfer
            Me.writeLine(source + ": " + relatedActivityId.ToString() + ": " + message, eventType.ToString(), id, eventCache.DateTime, Convert.ToInt32(eventCache.ThreadId), eventCache.ProcessId)
        End Sub

#End Region

#End Region

    End Class
End Namespace